@@ -91,6 +91,7 @@ static sidtab_t sidtab;
 extern policydb_t *policydbp;
 extern int mlspol;
 
+static int handle_unknown = SEPOL_DENY_UNKNOWN;
 static char *txtfile = "policy.conf";
 static char *binfile = "policy";
 
@@ -99,7 +100,7 @@ unsigned int policyvers = POLICYDB_VERSION_MAX;
 void usage(char *progname)
 {
 	printf
-	    ("usage:  %s [-b] [-d] [-M] [-c policyvers (%d-%d)] [-o output_file] [input_file]\n",
+	    ("usage:  %s [-b] [-d] [-U handle_unknown (allow,deny,reject) [-M] [-c policyvers (%d-%d)] [-o output_file] [input_file]\n",
 	     progname, POLICYDB_VERSION_MIN, POLICYDB_VERSION_MAX);
 	exit(1);
 }
@@ -390,7 +391,7 @@ int main(int argc, char **argv)
 	int show_version = 0;
 	struct policy_file pf;
 
-	while ((ch = getopt(argc, argv, "o:dbMVc:")) != EOF) {
+	while ((ch = getopt(argc, argv, "o:dbU:MVc:")) != EOF) {
 		switch (ch) {
 		case 'o':
 			outfile = optarg;
@@ -405,6 +406,20 @@ int main(int argc, char **argv)
 		case 'V':
 			show_version = 1;
 			break;
+		case 'U':
+			if (!strcasecmp(optarg, "deny")) {
+				handle_unknown = DENY_UNKNOWN;
+				break;
+			}
+			if (!strcasecmp(optarg, "allow")) {
+				handle_unknown = ALLOW_UNKNOWN;
+				break;
+			}
+			if (!strcasecmp(optarg, "reject")) {
+				handle_unknown = REJECT_UNKNOWN;
+				break;
+			}
+			usage(argv[0]);
 		case 'M':
 			mlspol = 1;
 			break;
@@ -474,6 +489,7 @@ int main(int argc, char **argv)
 				file, strerror(errno));
 			exit(1);
 		}
+		policy_file_init(&pf);
 		pf.type = PF_USE_MEMORY;
 		pf.data = map;
 		pf.len = sb.st_size;
@@ -515,6 +531,7 @@ int main(int argc, char **argv)
 
 		/* Let sepol know if we are dealing with MLS support */
 		parse_policy.mls = mlspol;
+		parse_policy.handle_unknown = handle_unknown;
 
 		policydbp = &parse_policy;
 
@@ -561,6 +578,7 @@ int main(int argc, char **argv)
 		policydb.policy_type = POLICY_KERN;
 		policydb.policyvers = policyvers;
 
+		policy_file_init(&pf);
 		pf.type = PF_USE_STDIO;
 		pf.fp = outfp;
 		ret = policydb_write(&policydb, &pf);
